GObject = {};
-- 
GObject.type = "draw"
GObject.name = ""
GObject.x = 0
GObject.y = 0
GObject.height = 0
GObject.width = 0
GObject.speed_x = 0
GObject.speed_y = 0
GObject.value = 0
GObject.color = 0
GObject.rect_color = 0
GObject.main_color = 0
GObject.sound = 0
GObject.timer = 0
GObject.timer_tick = 0
GObject.text_align = "left"
GObject.visible = 1
GObject.enable = true
GObject.selected_obj = 0
GObject.selected_obj_active = false
GObject.disabled_obj = 0
GObject.render_rect = false
GObject.mouse_entered = false
GObject.draw_cor_x = 0
GObject.draw_cor_y = 0
GObject.click_sound = 0
GObject.index = 1
GObject.anim_delay = 1
GObject.font = 0
GObject.dragging = {active = false, diffx = 0, diffy = 0, drag_min_x=0, drag_max_x=0, drag_min_y=0, drag_max_y=0}
GObject.dragtype = 0

--
GObject.on_timer = 0
GObject.left_button_click = 0
GObject.left_button_release = 0
GObject.mouse_enter = 0
GObject.mouse_leave = 0
GObject.on_custom_event = 0
GObject.on_update = 0
GObject.on_win_update = 0 -- update  ,    
GObject.on_drop = 0
GObject.on_drop_value = 0
GObject.on_init = 0
GObject.on_anim_end = 0
GObject.on_show = 0

-- 
GObject.when_stop = 0 --    (    )


function GObject.create(type)
local _gobject = {}
setmetatable(_gobject, {__index = GObject})
_gobject.type = type
return _gobject
end

function GObject:updateSpeed(dt)
	self.x=self.x+dt*self.speed_x
    self.y=self.y+dt*self.speed_y
    if self.when_stop~=0 then
    	if self.when_stop(self, self.speed_x, self.speed_y) then
    		self.speed_x = 0
    		self.speed_y = 0
    	end
    end
end
function GObject:setMyColor(color)	
	if color ~=0 then
		local r, g, b, a = love.graphics.getColor()
		self.main_color = {r, g, b, a}
		love.graphics.setColor(color)
	end
end
function GObject:restoreColor()
	if self.color ~=0 then
		love.graphics.setColor(self.main_color)
	end
end

function GObject:setVisible(make_visible)
	if make_visible then
		self.visible = 1
	else
		self.visible = 0
	end
end
function GObject:render()
	if self.visible==0 then
		return
	end
	if self.selected_obj_active and self.enable then
		self.selected_obj.visible=1
		self.selected_obj:render()
		self.selected_obj.visible=0
		return
	end
	if (not self.enable) and self.disabled_obj~=0 then
		self.disabled_obj.visible=1
		self.disabled_obj:render()
		self.disabled_obj.visible=0
		return
	end		
	
	self:setMyColor(self.color);	
	if self.type == "text" then
		local txtvalue = self.value
		if( type(txtvalue) == 'table') then
			txtvalue = self.value:get()
		end 
		local old_font = love.graphics.getFont()
		if self.font~=0 then
			love.graphics.setFont(self.font)
		end
		love.graphics.printf( txtvalue, self.x+self.draw_cor_x, self.y+self.draw_cor_y, self.width, self.text_align )
		if self.font~=0 then
			love.graphics.setFont(old_font)
		end
	end
	if self.type == "rectangle" then
		love.graphics.rectangle(self.value, self.x+self.draw_cor_x, self.y+self.draw_cor_y, self.width, self.height )
	end
	if self.type == "draw" then
		local draw_image = self.value
		if( type(draw_image) == 'table') then
			draw_image = self.value[self.index]
		end 		
		love.graphics.draw( draw_image, self.x+self.draw_cor_x, self.y+self.draw_cor_y, 0, (self.width/draw_image:getWidth()), (self.height/draw_image:getHeight()) )		
	end
	self:restoreColor()
end
function GObject:renderRect()
	if not self.render_rect or self.visible==0 then
		return
	end
	self:setMyColor(self.rect_color);
	love.graphics.rectangle("line", self.x, self.y, self.width, self.height )
	self:restoreColor()
end 

function GObject.renderArray(objs)
	for i=1,#objs do
		objs[i]:render()
	end
end
function GObject.renderRectArray(objs)
	for i=1,#objs do
		objs[i]:renderRect()
	end
end

function GObject.mousePressArray(objs, x, y, button)
	for i=1,#objs do
		objs[i]:mousepressed(x,y,button)
	end
end

function GObject.mouseReleaseArray(objs, x, y, button)
	for i=1,#objs do
		objs[i]:mousereleased(x,y,button)
	end
end

function GObject:IsPointCollision( x,y)
	return (x>=self.x and x<=(self.x+self.width) and y>=self.y and y<=(self.y+self.height))
end

function GObject:mousepressed(x,y,button)
	if (self.visible == 0) or (not self.enable) or ( not self:IsPointCollision(x,y) ) then
   		return
    end	
	if self.dragtype>0 then
   		self.dragging.active = true
   		self.dragging.diffx = x -- obj.parent.x
    	self.dragging.diffy = y -- obj.parent.y
    	self.bx = self.x
    	self.by = self.y
   end
   if self.click_sound~= 0 then
   		love.audio.stop(self.click_sound)
		love.audio.play(self.click_sound)
   end
   if (self.left_button_click == 0) then
   		return
   end	   
   if button == "l" then   	 
   	   self.left_button_click(self,x,y,button)	   
   end     
end

function GObject:mousereleased(x,y,button)  
   if(self.visible == 0) or (not self.enable) then
   		return
   end
   if self.dragtype>0 and self.dragging.active then   		   
   		if self.on_drop~=0 then	
    		self.on_drop(self, x, y,button)
    	end
    	self.bx = self.x
    	self.by = self.y
    	self.dragging.active = false
   end 	
   if(not self:IsPointCollision(x,y)) then
   		return
   end
   if (self.left_button_release == 0) then
   		return
   end
   if button == "l" then   	 
   	   self.left_button_release(self,x,y,button)	   
   end        
end

function GObject.findWithName(objs, name)
	for i=1,#objs do
		if objs[i].name == name then
			return objs[i]
		end
	end
	return nil
end

function GObject:setTimer(time, on_timer)
	self.timer = time
	self.on_timer = on_timer
	self.timer_tick = love.timer.getTime()
end 

function GObject:update(dt)	
	if (self.visible == 0) then
		return
	end
	if self.timer > 0 then
		local cur_tick = love.timer.getTime()
		if (cur_tick - self.timer_tick) >= self.timer then
			self.timer = 0
			if self.on_timer~=0 then
				self.on_timer(self)
			end
		end
	end
	if self.selected_obj~=0 then
		self.selected_obj.x = self.x
		self.selected_obj.y = self.y
		self.selected_obj.width = self.width
		self.selected_obj.height = self.height
		self.selected_obj_active = self:IsPointCollision(love.mouse.getX(), love.mouse.getY())
	end
	
	if (self.mouse_enter~=0) and (self:IsPointCollision(love.mouse.getX(), love.mouse.getY())) then
		self.mouse_enter(self)
		self.mouse_entered = true
	end
	
	if (self.mouse_leave~=0) and self.mouse_entered and (not self:IsPointCollision(love.mouse.getX(), love.mouse.getY())) then
		self.mouse_leave(self)
		self.mouse_entered = false
	end
	
	if self.on_update~=0 then
		self.on_update(self, dt)
	end
	
	if self.on_win_update~=0 then
		self.on_win_update(self, dt)
	end	
	
	if self.dragging.active then
		local old_x = self.x
		local old_y = self.y
		if self.dragtype == 1 or self.dragtype == 2 then
      		self.x = self.bx + (love.mouse.getX() - self.dragging.diffx)
      		if self.x<self.dragging.drag_min_x then
      			self.x = self.dragging.drag_min_x
      		end
      		if self.dragging.drag_max_x>0 and (self.x>self.dragging.drag_max_x) then
      			self.x = self.dragging.drag_max_x
      		end
      	end
      	if self.dragtype == 1 or self.dragtype == 3 then
      		self.y = self.by + (love.mouse.getY() - self.dragging.diffy)
      		
      		if self.y<self.dragging.drag_min_y then
      			self.y = self.dragging.drag_min_y
      		end
      		if self.dragging.drag_max_y>0 and (self.y>self.dragging.drag_max_y) then
      			self.y = self.dragging.drag_max_y
      		end
      	end
      	if self.on_drop_value~=0 then
      		self.on_drop_value(self,self.x,self.y)
      	end
   end   
end

function GObject:onShow()
	if self.on_show~=0 then
		self.on_show(self)
	end
	if self.selected_obj_active then
		self.selected_obj.x = self.x
		self.selected_obj.y = self.y
	end
	if self.disabled_obj~=0 then
		self.disabled_obj.x = self.x
		self.disabled_obj.y = self.y
	end
end

function GObject.selectRect(obj)
	obj.render_rect=true
end

function GObject.unselectRect(obj)
	obj.render_rect=false
end

function GObject:updateArray(objs, dt)
	for i=1,#objs do
		objs[i]:update(dt)
		objs[i]:updateSpeed(dt)		
	end
end

function GObject:resetTimer()
	self.timer = 0
end

function GObject:setSelected(filename)
	if not filename then
		filename = self.name+'_selected'
	end
	if (not file_exists(filename)) then
		return
	end
	local button_selected = GObject.create("draw")
    button_selected.x = self.x
    button_selected.y = self.y
    button_selected.value = love.graphics.newImage(filename)
    button_selected.visible=0
    button_selected.width = self.width
    button_selected.height = self.height   
    self.selected_obj = button_selected
end

function file_exists(name)
   return love.filesystem.exists(name)
end

function GObject:setDisabled(filename)
	if not filename then
		filename = self.name+'_disabled'
	end
	if (not file_exists(filename)) then		
		return
	end	
	local button_disabled = GObject.create("draw")
    button_disabled.x = self.x
    button_disabled.y = self.y
    button_disabled.value = love.graphics.newImage(filename)
    button_disabled.visible=0
    button_disabled.width = self.width
    button_disabled.height = self.height   
    self.disabled_obj = button_disabled
end

function GObject:loadButton(name)
	self.value = love.graphics.newImage(name..'.png')
	if self.height==0 then
		self.height = transformSize(self.value:getHeight())
	end
	if self.width==0 then
		self.width = transformSize(self.value:getWidth())
	end
	self.name = name
	self:setSelected(name.."-selected.png")
	self:setDisabled(name.."-disabled.png")
end

function transformSize(size)
	return (Configuration.inst().screen_width/1024*size)
end

function GObject:setText(text)
	local value = glang:get(text)
	if value == nil then
		value = text
	end
	self.value = value	
end

function GObject:loadDraw(name)
	local full_name = name..".png"
	self.value = love.graphics.newImage(full_name)
	if self.height==0 then
		self.height = transformSize(self.value:getHeight())
	end
	if self.width==0 then
		self.width = transformSize(self.value:getWidth())
	end
end

function GObject:loadAnimation(name, count, anim_delay)
	self.value = {}
	for i=1,count do
		local full_name = name..i..'.png'
		local img = love.graphics.newImage(full_name)
		table.insert(self.value, img)
	end	
    self.height=transformSize(self.value[1]:getHeight())
    self.width=transformSize(self.value[1]:getWidth())
    self.anim_delay = anim_delay
end
function GObject:startAnimation(start_index, end_function)
	if start_index~=nil then
		self.index = start_index
	end
	if end_function~=nil then
		self.on_anim_end = end_function
	end
	self:setTimer(self.anim_delay, GObject.next_cadre)
	--value
end
function GObject.next_cadre(obj)
	if obj.index<#obj.value then
		obj.index=obj.index+1
		obj:setTimer(obj.anim_delay, GObject.next_cadre)
	else
		if( obj.on_anim_end~=0 ) then
			obj.on_anim_end(obj)
		end
	end
end

function GObject.anim_loop(obj)
	obj:startAnimation(1,GObject.anim_loop)
end

function GObject:addToSceneManager(man_objs)
	table.insert(man_objs, self)
end

function typeof(var)
    local _type = type(var);
    if(_type ~= "table" and _type ~= "userdata") then
        return _type;
    end
    local _meta = getmetatable(var);
    if(_meta ~= nil and _meta._NAME ~= nil) then
        return _meta._NAME;
    else
        return _type;
    end
end

function GObject:setParentAndName(parent, name)
	self.parent = parent
	self.name = parent.name..'.'..name
end
